import logging

from .manager import pipelines

# pylint: disable=unused-import
from .celery_tasks import celery_app, execute_pipeline_task


for pipeline_id, pipeline_details, pipeline_cwd, dirty \
        in pipelines():
    if dirty:
        logging.info('Executing DIRTY task %s', pipeline_id)
        execute_pipeline_task.delay(pipeline_id,
                                    pipeline_details['pipeline'],
                                    pipeline_cwd)



import logging

from datapackage_pipelines.wrapper import ingest, spew

params, datapackage, res_iter = ingest()

column_aliases = params['column-aliases']
column_mapping = {}
for target, sources in column_aliases.items():
    if sources is not None:
        for source in sources:
            if source in column_mapping:
                logging.error('Duplicate appearance of %s', source)
            assert source not in column_mapping
            column_mapping[source] = target
    if target in column_mapping:
        logging.error('Duplicate appearance of %s', target)
    assert target not in column_mapping
    column_mapping[target] = target

resource_name = params.get('resource-name', 'concat')
concat_resource = {
    'name': resource_name,
    'path': 'data/'+resource_name+'.csv',
    'mediatype': 'text/csv',
    'schema': {
        'fields': [],
        'primaryKey': []
    },
}

used_fields = set()
for resource in datapackage['resources']:
    schema = resource.get('schema', {})
    pk = schema.get('primaryKey', [])
    for field in schema.get('fields', []):
        orig_name = field['name']
        if orig_name in column_mapping:
            name = column_mapping[orig_name]
            if name in used_fields:
                continue
            if orig_name in pk:
                concat_resource['schema']['primaryKey'].append(name)
            concat_resource['schema']['fields'].append(field)
            field['name'] = name
            used_fields.add(name)

datapackage['resources'] = [concat_resource]


def process_resources(_res_iter):
    for rows_iter in _res_iter:
        for row in rows_iter:
            processed = dict((k, '') for k in used_fields)
            values = [(column_mapping[k], v) for (k, v)
                      in row.items()
                      if k in column_mapping]
            assert len(values) > 0
            processed.update(dict(values))
            yield processed

spew(datapackage, [process_resources(res_iter)])



import os
import csv
import logging

import itertools
import requests

from jsontableschema.model import SchemaModel

from datapackage_pipelines.wrapper import ingest, spew


def _reader(opener, _url):
    yield None
    filename = os.path.basename(_url)
    _schema, _headers, _csv_reader = opener()
    i = 0
    for i, row in enumerate(_csv_reader):

        row = [x.strip() for x in row]
        if len(row) == 0:
            # response.iter_lines() might emit an empty string here and there
            # if the delimiter is more than one byte:
            #       https://github.com/kennethreitz/requests/pull/2431
            continue
        values = set(row)
        if len(values) == 1 and '' in values:
            # In case of empty rows, just skip them
            continue
        output = dict(
            (header, _schema.cast(header, value))
            for header, value
            in zip(_headers, row)
            if _schema.has_field(header)
        )
        yield output

        i += 1
        if i % 10000 == 0:
            logging.info('%s: %d rows', filename, i)
            # break

    logging.info('%s: TOTAL %d rows', filename, i)


def _null_remover(iterator):
    for line in iterator:
        if line == '':
            continue
        if '\x00' in line:
            continue
        yield line


def csv_stream_reader(_resource, _url, _encoding=None):
    def get_opener(__url, __encoding):
        def opener():
            if __url.startswith('file://'):
                response = open(__url[7:], encoding=__encoding)
            else:
                response = requests.get(__url, stream=True)
                if __encoding is not None:
                    response.encoding = __encoding
                response = response.iter_lines(decode_unicode=True)
            _csv_reader = csv.reader(_null_remover(response))
            _headers = next(_csv_reader)
            _schema = _resource.get('schema')
            if _schema is not None:
                _schema = SchemaModel(_schema)
            return _schema, _headers, _csv_reader
        return opener

    schema, headers, csv_reader = get_opener(url, _encoding)()
    if schema is None:
        schema = {
            'fields': [
                {'name': header, 'type': 'string'}
                for header in headers
                ]
        }
        _resource['schema'] = schema
    del csv_reader

    return itertools\
        .islice(
            _reader(
                get_opener(_url, _encoding),
                _url),
            1, None)


params, datapackage, res_iter = ingest()

new_resource_iterators = []
for resource in datapackage['resources']:
    if 'path' in resource:
        new_resource_iterators.append(next(res_iter))
    elif 'url' in resource and resource.get('mediatype') == 'text/csv':
        url = resource['url']
        basename = os.path.basename(resource['url'])
        path = os.path.join('data', basename)
        del resource['url']
        resource['path'] = path
        encoding = resource.get('encoding')
        rows = csv_stream_reader(resource, url, encoding)
        new_resource_iterators.append(rows)

spew(datapackage, new_resource_iterators)



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

import io
import os

VERSION_FILE = os.path.join(os.path.dirname(__file__), 'VERSION')

__version__ = io.open(VERSION_FILE, encoding='utf-8').readline().strip()

# __all__ = ['initialize_pipeline']



import os
import zipfile
import csv
import json
import tempfile
import logging

from jsontableschema.exceptions import InvalidCastError
from jsontableschema.model import SchemaModel

from datapackage_pipelines.wrapper import ingest, spew

params, datapackage, res_iter = ingest()

out_filename = open(params['out-file'], 'wb')
out_file = zipfile.ZipFile(out_filename, 'w')

out_file.writestr('datapackage.json',
                  json.dumps(datapackage, ensure_ascii=True))


def transform_value(value, _):
    if value is None:
        return ''
    return str(value)


def transform_row(row, fields):
    return dict((k, transform_value(v, fields[k]['type']))
                for k, v in row.items())


def rows_processor(_rows, _csv_file, _zip_file, _writer, _fields, _schema):
    for row in _rows:
        transformed_row = transform_row(row, _fields)
        for k, v in transformed_row.items():
            try:
                _schema.cast(k, v)
            except InvalidCastError:
                logging.error('Bad value %r for field %s', v, k)
                raise
        _writer.writerow(transformed_row)
        yield row
    filename = _csv_file.name
    _csv_file.close()
    _zip_file.write(filename, arcname=_rows.spec['path'],
                    compress_type=zipfile.ZIP_DEFLATED)
    os.unlink(filename)


def resource_processor(_res_iter, zip_file):
    for rows in _res_iter:
        schema = rows.spec['schema']

        csv_file = tempfile.NamedTemporaryFile(mode="w+", delete=False)
        fields = schema['fields']
        headers = list(map(lambda field: field['name'], fields))

        csv_writer = csv.DictWriter(csv_file, headers)
        csv_writer.writeheader()

        fields = dict((field['name'], field) for field in fields)
        schema = SchemaModel(schema)

        yield rows_processor(rows, csv_file, zip_file,
                             csv_writer, fields, schema)

spew(datapackage, resource_processor(res_iter, out_file))
out_file.close()



from .celery_app import celery_app

from ..manager.tasks import execute_pipeline


@celery_app.task
def execute_pipeline_task(pipeline_id, pipeline_steps, pipeline_cwd):
    execute_pipeline(pipeline_id,
                     pipeline_steps,
                     pipeline_cwd,
                     'schedule',
                     False)



from datapackage_pipelines.wrapper import ingest, spew

params, datapackage, res_iter = ingest()

columns = params['columns']

for resource in datapackage['resources']:
    fields = resource.get('schema', {}).get('fields')
    if fields is not None:
        fields = [
            field for field in fields
            if field['name'] not in columns
        ]
        resource['schema']['fields'] = fields


def process_resources(_res_iter):
    for rows in _res_iter:
        def process_rows(_rows):
            for row in _rows:
                for column in columns:
                    if column in row:
                        del row[column]
                yield row
        yield process_rows(rows)

spew(datapackage, process_resources(res_iter))



import os
import zipfile
import tempfile

from datapackage_pipelines.wrapper import ingest, spew

import gobble

params, datapackage, res_iter = ingest()
spew(datapackage, res_iter)

user = gobble.user.User()
in_filename = open(params['in-file'], 'rb')

in_file = zipfile.ZipFile(in_filename)
temp_dir = tempfile.mkdtemp()
for name in in_file.namelist():
    in_file.extract(name, temp_dir)
in_file.close()
datapackage_json = os.path.join(temp_dir, 'datapackage.json')

package = gobble.fiscal.FiscalDataPackage(datapackage_json, user=user)
package.upload(skip_validation=True)



import os

from celery import Celery
from celery.schedules import crontab

from datapackage_pipelines.manager import pipelines

TASK_NAME = 'datapackage_pipelines.celery_tasks.celery_tasks' + \
                '.execute_pipeline_task'


CELERY_SCHEDULE = {}

for pipeline_id, pipeline_details, pipeline_cwd, dirty \
        in pipelines():
    entry = {
        'task': TASK_NAME,
        'schedule': crontab(*pipeline_details['schedule']),
        'args': (pipeline_id,
                 pipeline_details['pipeline'],
                 pipeline_cwd)
    }
    CELERY_SCHEDULE[pipeline_id] = entry

celery_app = Celery('dpp')
celery_app.conf.update(CELERYBEAT_SCHEDULE=CELERY_SCHEDULE,
                       CELERY_TIMEZONE='UTC',
                       CELERY_REDIRECT_STDOUTS=False,
                       BROKER_URL=os.environ.get('CELERY_BROKER', 'amqp://'),
                       CELERY_RESULT_BACKEND=os.environ.get('CELERY_BACKEND',
                                                            'amqp://'),
                       CELERYD_LOG_LEVEL="INFO",
                       CELERY_TASK_SERIALIZER='json',
                       CELERY_RESULT_SERIALIZER='json',
                       CELERY_ACCEPT_CONTENT=['json'],
                       CELERY_ROUTES={
                           TASK_NAME: {'queue': 'datapackage-pipelines'},
                       })



import datetime
import json
import copy
import decimal
import logging

import datapackage
from jsontableschema.exceptions import InvalidCastError

from jsontableschema.model import SchemaModel


class CommonJSONDecoder(json.JSONDecoder):
    """
    Common JSON Encoder
    json.loads(myString, cls=CommonJSONEncoder)
    """

    @classmethod
    def object_hook(cls, obj):  # pylint: disable=method-hidden
        if 'type{decimal}' in obj:
            try:
                return decimal.Decimal(obj['type{decimal}'])
            except decimal.InvalidOperation:
                pass
        if 'type{date}' in obj:
            try:
                return datetime.datetime \
                    .strptime(obj["type{date}"], '%Y-%m-%d') \
                    .date()
            except ValueError:
                pass

        return obj

    def __init__(self, **kwargs):
        kwargs['object_hook'] = self.object_hook
        super(CommonJSONDecoder, self).__init__(**kwargs)


# pylint: disable=too-few-public-methods, too-many-arguments
class ResourceIterator(object):

    def __init__(self, infile, spec, orig_spec,
                 validate=False, debug=False):
        self.spec = spec
        self.table_schema = SchemaModel(orig_spec['schema'])
        self.validate = validate
        self.infile = infile
        self.debug = debug

    def __iter__(self):
        return self

    def __next__(self):
        if self.debug:
            logging.error('WAITING')
        line = self.infile.readline().strip()
        if self.debug:
            logging.error('INGESTING: %r', line)
        if line == '':
            raise StopIteration()
        line = json.loads(line, cls=CommonJSONDecoder)
        if self.validate:
            for k, v in line.items():
                try:
                    self.table_schema.cast(k, v)
                except (InvalidCastError, TypeError):
                    field = self.table_schema.get_field(k)
                    if field is None:
                        raise ValueError('Validation failed: No such field %s',
                                         k)
                    else:
                        raise ValueError('Validation failed: Bad value %r '
                                         'for field %s with type %s',
                                         v, k, field.get('type'))

        return line

    def next(self):
        return self.__next__()


def process_input(infile, validate=False, debug=False):

    dp_json = infile.readline().strip()
    if dp_json == '':
        raise ValueError('Missing input')
    dp = json.loads(dp_json)
    resources = dp.get('resources', [])
    original_resources = copy.deepcopy(resources)

    profiles = list(dp.get('profiles', {}).keys())
    profile = 'tabular'
    if 'tabular' in profiles:
        profiles.remove('tabular')
    if len(profiles) > 0:
        profile = profiles.pop(0)
    schema = datapackage.schema.Schema(profile)
    schema.validate(dp)

    _ = infile.readline().strip()

    def resources_iterator(_resources, _original_resources):
        # we pass a resource instance that may be changed by the processing
        # code, so we must keep a copy of the original resource (used to
        # validate incoming data)
        for resource, orig_resource in zip(_resources, _original_resources):
            if 'path' not in resource:
                continue

            res_iter = ResourceIterator(infile,
                                        resource, orig_resource,
                                        validate, debug)
            yield res_iter

    return dp, resources_iterator(resources, original_resources)



import json
import logging
import subprocess

from datapackage_pipelines.wrapper import ingest, spew

params, datapackage, res_iter = ingest()

os_types = params['os-types']
options = params['options']

resource = datapackage['resources'][0]
fields = resource['schema']['fields']

for field in fields:
    field_name = field['name']
    if field_name not in os_types:
        logging.error('Missing OS Type for field %s', field_name)
    field['type'] = os_types[field_name]
    field['options'] = options.get(field_name, {})

result = subprocess.run(['/usr/bin/env', 'os-types', json.dumps(fields)],
                        stdout=subprocess.PIPE, stderr=subprocess.PIPE)

errors = result.stderr.decode('utf8')
if len(errors) > 0:
    raise RuntimeError(errors)
output = result.stdout.decode('utf8')
if output.startswith('FAILED'):
    raise RuntimeError(output)

model = json.loads(output)

datapackage['model'] = model['model']

for field in fields:
    field.update(model['schema']['fields'][field['name']])
    if 'options' in field:
        del field['options']

resource['schema']['primaryKey'] = model['schema']['primaryKey']

datapackage['profiles'] = {
    'fiscal': '*',
    'tabular': '*'
}
datapackage['@context'] = \
    'http://schemas.frictionlessdata.io/fiscal-data-package.jsonld'

spew(datapackage, res_iter)



import click

from .manager import execute_pipeline
from .manager.status import status
from .manager.specs import pipelines


@click.group(invoke_without_command=True)
@click.pass_context
def cli(ctx):
    if ctx.invoked_subcommand is None:
        click.echo('Available Pipelines:')
        for pipeline_id, _, _, dirty in pipelines():
            click.echo('- {} {}'.format(pipeline_id, '(*)' if dirty else ''))


@cli.command()
def serve():
    """Start the web server"""
    from .web import app
    app.run(host='0.0.0.0', debug=True, port=5000)


@cli.command()
@click.argument('pipeline_id')
@click.option('--use-cache/--no-use-cache', default=True)
def run(pipeline_id, use_cache):
    """Execute a single pipeline"""
    for _pipeline_id, pipeline_details, pipeline_cwd, _ \
            in pipelines():
        if _pipeline_id == pipeline_id:
            execute_pipeline(pipeline_id,
                             pipeline_details['pipeline'],
                             pipeline_cwd,
                             use_cache=use_cache)
            break


@cli.command()
def init():
    """Reset the status of all pipelines"""
    status.initialize()



import json
import os
import logging
import hashlib

import yaml

from .status import status

SPEC_FILENAME = 'pipeline-spec.yaml'
PROCESSOR_PATH = os.environ.get('DATAPIPELINES_PROCESSOR_PATH', '').split(';')


def find_specs(root_dir='.'):
    for dirpath, _, filenames in os.walk(root_dir):
        if SPEC_FILENAME in filenames:
            abspath = os.path.abspath(dirpath)
            fullpath = os.path.join(abspath, SPEC_FILENAME)
            with open(fullpath, encoding='utf8') as spec_file:
                spec = yaml.load(spec_file.read())
                for pipeline_id, pipeline_details in spec.items():
                    pipeline_id = os.path.join(dirpath, pipeline_id)
                    yield abspath, pipeline_id, pipeline_details


def resolve_executor(executor, path):

    parts = []
    while executor.startswith('..'):
        parts.append('..')
        executor = executor[1:]

    if executor.startswith('.'):
        executor = executor[1:]

    executor = executor.split('.')
    executor[-1] += '.py'

    parts.extend(executor)

    local_parts = [path] + parts
    resolve_order = [local_parts]

    for resolve_location in PROCESSOR_PATH:
        resolve_order.append([os.path.abspath(resolve_location)] + parts)

    lib_parts = [os.path.dirname(__file__), '..', 'lib'] + parts
    resolve_order.append(lib_parts)

    for option in resolve_order:
        option = os.path.join(*option)
        if os.path.exists(option):
            return option

    raise FileNotFoundError("Couldn't resolve {0} at {1}"
                            .format(executor, path))


def validate_required_keys(obj, keys, abspath):
    for required_key in keys:
        if required_key not in obj:
            raise KeyError('Missing parameter {0} in {1}'
                           .format(required_key, abspath))


def validate_specs():

    all_pipeline_ids = set()

    for abspath, pipeline_id, pipeline_details in find_specs():

        if pipeline_id in all_pipeline_ids:
            raise KeyError('Duplicate key {0} in {1}'
                           .format(pipeline_id, abspath))

        validate_required_keys(pipeline_details,
                               ['pipeline', 'schedule'],
                               abspath)

        pipeline = pipeline_details['pipeline']

        try:
            for step in pipeline:
                validate_required_keys(step, ['run'], abspath)
                executor = step['run']
                step['name'] = executor
                step['run'] = resolve_executor(executor, abspath)
        except FileNotFoundError as e:
            logging.error(e)
            continue

        cache_hash = ''
        for step in pipeline:
            m = hashlib.md5()
            m.update(cache_hash.encode('ascii'))
            m.update(open(step['run'], 'rb').read())
            m.update(json.dumps(step, ensure_ascii=True, sort_keys=True)
                     .encode('ascii'))
            cache_hash = m.hexdigest()
            step['_cache_hash'] = cache_hash

        schedule = pipeline_details['schedule']
        if 'crontab' in schedule:
            schedule = schedule['crontab'].split()
            pipeline_details['schedule'] = schedule
        else:
            raise NotImplementedError("Couldn't find valid schedule at {0}"
                                      .format(abspath))

        dirty = status.register(pipeline_id, cache_hash)

        yield pipeline_id, pipeline_details, abspath, dirty


def pipelines():
    return validate_specs()



import datetime

from flask import Flask, render_template, abort, redirect

from ..manager.status import status

app = Flask(__name__)


def datestr(x):
    return str(datetime.datetime.fromtimestamp(x))


@app.route("/")
def main():
    statuses = status.all_statuses()
    for pipeline in statuses:
        for key in ['ended', 'last_success', 'started']:
            if pipeline.get(key):
                pipeline[key] = datestr(pipeline[key])
        pipeline['class'] = 'warning' if pipeline['running'] else \
            'success' if pipeline.get('success') \
            else 'danger'
        pipeline['status'] = 'Running' if pipeline['running'] else \
            'Idle' if pipeline.get('success') \
            else 'Failed'
        pipeline['slug'] = pipeline['id'].replace('/', '_').replace('.', '_')
    return render_template('dashboard.html', pipelines=statuses)


@app.route("/badge/<path:pipeline_id>")
def badge(pipeline_id):
    if not pipeline_id.startswith('./'):
        pipeline_id = './' + pipeline_id
    pipeline_status = status.get_status(pipeline_id)
    if pipeline_status is None:
        abort(404)
    success = pipeline_status.get('success')
    if success is True:
        status_text = 'Passing'
        record_count = pipeline_status.get('record_count')
        if record_count is not None:
            status_text += ' (%d records)' % record_count
        status_color = 'brightgreen'
    elif success is False:
        status_text = 'Failing'
        status_color = 'red'
    else:
        status_text = "Haven't run"
        status_color = 'lightgray'
    return redirect('https://img.shields.io/badge/{}-{}-{}.svg'.format(
        'pipeline', status_text, status_color
    ))



import gzip
import sys
import os
import json
import logging
import decimal
import datetime

from .input_processor import process_input


def processor():
    return "%-32s" % os.path.basename(sys.argv[0]).split('.')[0].title()

logging.basicConfig(level=logging.DEBUG,
                    format="%(levelname)-8s:"+processor()+":%(message)s")


class CommonJSONEncoder(json.JSONEncoder):
    """
    Common JSON Encoder
    json.dumps(myString, cls=CommonJSONEncoder)
    """

    def default(self, obj):     # pylint: disable=method-hidden

        if isinstance(obj, decimal.Decimal):
            return {'type{decimal}': str(obj)}
        elif isinstance(obj, datetime.date):
            return {'type{date}': str(obj)}


cache = ''


def ingest(debug=False):
    global cache # pylint: disable=global-statement
    params = None
    first = True
    validate = False
    if len(sys.argv) > 4:
        first = sys.argv[1] == '0'
        params = json.loads(sys.argv[2])
        validate = sys.argv[3] == 'True'
        cache = sys.argv[4]

    if first:
        return params, None, None

    datapackage, resource_iterator = process_input(sys.stdin, validate, debug)
    return params, datapackage, resource_iterator


def spew(dp, resources_iterator):
    files = [sys.stdout]

    cache_filename = ''
    if len(cache) > 0:
        if not os.path.exists('.cache'):
            os.mkdir('.cache')
        cache_filename = os.path.join('.cache', cache)
        files.append(gzip.open(cache_filename+'.ongoing', 'wt'))

    row_count = 0
    for f in files:
        f.write(json.dumps(dp, ensure_ascii=True)+'\n')
    for res in resources_iterator:
        for f in files:
            f.write('\n')
        for rec in res:
            line = json.dumps(rec, cls=CommonJSONEncoder, ensure_ascii=True)
            # logging.error('SPEWING: {}'.format(line))
            for f in files:
                f.write(line+'\n')
            # logging.error('WROTE')
            row_count += 1

    logging.info('Processed %d rows', row_count)

    for f in files:
        f.close()

    if len(cache) > 0:
        os.rename(cache_filename+'.ongoing', cache_filename)



import json
import logging
import os
import time

import redis


class RedisConnection(object):

    def __init__(self, host=None, port=6379):
        if host is not None and len(host) > 0:
            self.redis = redis.StrictRedis(host=host, port=port, db=5)
            try:
                self.redis.ping()
            except redis.exceptions.ConnectionError:
                logging.warning('Failed to connect to Redis, host:%s, port:%s',
                                host, port)
                self.redis = None
        else:
            logging.warning('Failed to connect to Redis, host:%s, port:%s',
                            host, port)
            self.redis = None

    def is_running(self, _id):
        if self.redis is None:
            return False
        _status = self.redis.get(_id)
        _status = json.loads(_status.decode('ascii'))
        cur_time = time.time()
        # A running task must have been updated in the last 10 seconds
        is_running = \
            (_status['running'] is True) and \
            (cur_time - _status.get('updated', cur_time) < 60)
        return is_running

    def running(self, _id, trigger=None, log=None):
        if self.redis is None:
            return True
        _status = self.redis.get(_id)
        _status = json.loads(_status.decode('ascii'))

        cur_time = time.time()
        if not self.is_running(_id):
            _status['started'] = cur_time

        _status.update({
            'id': _id,
            'running': True,
            'trigger': trigger,
            'updated': cur_time,
            'reason': log
        })
        self.redis.set(_id, json.dumps(_status))

    def idle(self, _id, success, reason=None, cache_hash=None,
             record_count=None):
        if self.redis is None:
            return
        _status = self.redis.get(_id)
        _status = json.loads(_status.decode('ascii'))
        cur_time = time.time()
        _status.update({
            'id': _id,
            'running': False,
            'ended': cur_time,
            'updated': cur_time,
            'success': success,
            'reason': reason
        })
        if success is True:
            _status.update({
                'last_success': _status['ended'],
                'cache_hash': cache_hash,
                'record_count': record_count
            })
        self.redis.set(_id, json.dumps(_status, ensure_ascii=True))

    def register(self, _id, cache_hash):
        if self.redis is None:
            return
        self.redis.sadd('all-pipelines', _id)
        _status = self.redis.get(_id)
        if _status is None:
            _status = {
                'id': _id,
                'cache_hash': ''
            }
        else:
            _status = json.loads(_status.decode('ascii'))
        dirty = _status.get('cache_hash') != cache_hash
        _status.update({
            'running': False,
        })
        self.redis.set(_id, json.dumps(_status, ensure_ascii=True))
        return dirty

    def initialize(self):
        if self.redis is None:
            return
        self.redis.delete('all-pipelines')

    def get_status(self, _id):
        if self.redis is None:
            return None
        sts = self.redis.get(_id)
        if sts is not None:
            return json.loads(sts.decode('ascii'))
        else:
            return None

    def all_statuses(self):
        if self.redis is None:
            return []
        all_ids = self.redis.smembers('all-pipelines')
        pipe = self.redis.pipeline()
        for _id in all_ids:
            pipe.get(_id)
        return [json.loads(sts.decode('ascii')) for sts in pipe.execute()]

status = RedisConnection(os.environ.get('DATAPIPELINES_REDIS_HOST'))
